@@ -4,13 +4,25 @@
   This is the only file that must be included to use the CDI library from C.
 */
 
-#ifndef  CDI_H_
-#define  CDI_H_
+#ifndef CDI_H_
+#define CDI_H_
+
+// clang-format off
 
 #include <stdio.h>
 #include <stdint.h>    // int64_t
+#include <stdbool.h>
 #include <sys/types.h>
 
+#ifndef CDI_SIZE_TYPE
+#define CDI_SIZE_TYPE int
+#endif
+
+typedef CDI_SIZE_TYPE SizeType;
+
+//FINT_ON  <--- don't change or remove this line!!!
+// Start of fortran interface for the following routines (make_fint.c)
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -46,6 +58,8 @@ extern "C" {
 #define  CDI_EUFSTRUCT            -25   // Unsupported file structure
 #define  CDI_EUNC4                -26   // Unsupported NetCDF4 structure
 #define  CDI_EDIMSIZE             -27   // Invalid dimension size
+#define  CDI_EQENF                -50   // Query entries not found
+#define  CDI_EQNAVAIL             -51   // Query not available for file type
 #define  CDI_ELIMIT               -99   // Internal limits exceeded
 
 // File types
@@ -60,6 +74,21 @@ extern "C" {
 #define  CDI_FILETYPE_SRV           8   // File type SERVICE
 #define  CDI_FILETYPE_EXT           9   // File type EXTRA
 #define  CDI_FILETYPE_IEG          10   // File type IEG
+#define  CDI_FILETYPE_NCZARR       11   // File type NetCDF NCZarr Data Model
+
+// Compatibility defines for release 1.8.3 (obsolete defines)
+#define  FILETYPE_GRB               1   // File type GRIB
+#define  FILETYPE_GRB2              2   // File type GRIB version 2
+#define  FILETYPE_NC                3   // File type NetCDF
+#define  FILETYPE_NC2               4   // File type NetCDF version 2 (64-bit offset)
+#define  FILETYPE_NC4               5   // File type NetCDF version 4
+
+// Protocols (in filename/URI)
+
+#define CDI_PROTOCOL_OTHER          0   // Any other protocol (might be supported by NetCDF library)
+#define CDI_PROTOCOL_FILE           1
+#define CDI_PROTOCOL_FDB            2
+#define CDI_PROTOCOL_ACROSS         3
 
 // Compress types
 
@@ -68,9 +97,18 @@ extern "C" {
 #define  CDI_COMPRESS_AEC           2
 #define  CDI_COMPRESS_ZIP           3
 #define  CDI_COMPRESS_JPEG          4
+#define  CDI_COMPRESS_FILTER        5
 
 // external data types
 
+// Compatibility defines for release 1.8.3 (obsolete defines)
+#define  DATATYPE_PACK16           16
+#define  DATATYPE_PACK24           24
+#define  DATATYPE_FLT32           132
+#define  DATATYPE_FLT64           164
+#define  DATATYPE_INT32           232
+#define  DATATYPE_INT             251
+
 #define  CDI_DATATYPE_PACK          0
 #define  CDI_DATATYPE_PACK1         1
 #define  CDI_DATATYPE_PACK2         2
@@ -124,7 +162,7 @@ extern "C" {
 #define  CDI_DATATYPE_LONG        256
 #define  CDI_DATATYPE_UINT        257
 
-// Chunks
+// Chunk types
 
 #define  CDI_CHUNK_AUTO             1  // use default chunk size
 #define  CDI_CHUNK_GRID             2
@@ -150,6 +188,7 @@ extern "C" {
 #define  CDI_PROJ_LAEA             23  // Lambert Azimuthal Equal Area
 #define  CDI_PROJ_SINU             24  // Sinusoidal
 #define  CDI_PROJ_STERE            25  // Polar stereographic
+#define  CDI_PROJ_HEALPIX          26  // Healpix
 
 // ZAXIS types
 
@@ -210,6 +249,7 @@ typedef struct  {
 
 // TSTEP types
 
+#define  TSTEP_CONSTANT           0  // obsolete, use TIME_CONSTANT
 #define  TSTEP_INSTANT            1  // Instant
 #define  TSTEP_AVG                2  // Average
 #define  TSTEP_ACCUM              3  // Accumulation
@@ -272,7 +312,6 @@ typedef struct CdiGribIterator CdiGribIterator;
 void    cdiReset(void);
 
 const char *cdiStringError(int cdiErrno);
-
 void    cdiDebug(int debug);
 
 const char *cdiLibraryVersion(void);
@@ -302,8 +341,8 @@ int     cdiEncodeParam(int pnum, int pcat, int pdis);
 // date format:  YYYYMMDD
 // time format:    hhmmss
 
-void    cdiDecodeDate(int64_t date, int *year, int *month, int *day);
-int64_t cdiEncodeDate(int year, int month, int day);
+void    cdiDecodeDate(int date, int *year, int *month, int *day);
+int     cdiEncodeDate(int year, int month, int day);
 
 void    cdiDecodeTime(int time, int *hour, int *minute, int *second);
 int     cdiEncodeTime(int hour, int minute, int second);
@@ -311,7 +350,12 @@ int     cdiEncodeTime(int hour, int minute, int second);
 
 // STREAM control routines
 
-int     cdiGetFiletype(const char *path, int *byteorder);
+int     cdiGetFiletype(const char *uri, int *byteorder);
+int     cdiGetProtocol(const char *uri, const char **filename);
+
+int     streamOpenReadNCMem(int ncid);
+int     streamOpenWriteNCMem(int ncid);
+void    streamCloseNCMem(int ncid);
 
 //      streamOpenRead: Open a dataset for reading
 int     streamOpenRead(const char *path);
@@ -327,8 +371,11 @@ void    streamClose(int streamID);
 //      streamSync: Synchronize an Open Dataset to Disk
 void    streamSync(int streamID);
 
+void    streamDefMaxSteps(int streamID, int maxSteps);
 void    streamDefNumWorker(int streamID, int numWorker);
 
+int     streamInqNumSteps(int streamID);
+
 //      streamDefVlist: Define the Vlist for a stream
 void    streamDefVlist(int streamID, int vlistID);
 
@@ -344,6 +391,10 @@ void    streamDefByteorder(int streamID, int byteorder);
 //      streamInqByteorder: Get the byteorder
 int     streamInqByteorder(int streamID);
 
+void    streamDefShuffle(int streamID, int shuffle);
+
+void    streamDefFilter(int streamID, const char *filterSpec);
+
 //      streamDefCompType: Define compression type
 void    streamDefCompType(int streamID, int comptype);
 
@@ -368,42 +419,43 @@ int     streamInqCurTimestepID(int streamID);
 const char *streamFilename(int streamID);
 const char *streamFilesuffix(int filetype);
 
-size_t  streamNvals(int streamID);
+SizeType streamNvals(int streamID);
 
 int     streamInqNvars(int streamID);
 
 // STREAM var I/O routines (random access)
 
 //      streamWriteVar: Write a variable
-void    streamWriteVar(int streamID, int varID, const double data[], size_t nmiss);
-void    streamWriteVarF(int streamID, int varID, const float data[], size_t nmiss);
+void    streamWriteVar(int streamID, int varID, const double data[], SizeType numMissVals);
+void    streamWriteVarF(int streamID, int varID, const float data[], SizeType numMissVals);
 
 //      streamReadVar: Read a variable
-void    streamReadVar(int streamID, int varID, double data[], size_t *nmiss);
-void    streamReadVarF(int streamID, int varID, float data[], size_t *nmiss);
-void    streamReadVarPart(int streamID, int varID, int varType, int start, size_t size, void *data, size_t *nmiss, int memtype);
+void    streamReadVar(int streamID, int varID, double data[], SizeType *numMissVals);
+void    streamReadVarF(int streamID, int varID, float data[], SizeType *numMissVals);
+void    streamReadVarPart(int streamID, int varID, int varType, int start, SizeType size, void *data, SizeType *numMissVals, int memtype);
 
 //      streamWriteVarSlice: Write a horizontal slice of a variable
-void    streamWriteVarSlice(int streamID, int varID, int levelID, const double data[], size_t nmiss);
-void    streamWriteVarSliceF(int streamID, int varID, int levelID, const float data[], size_t nmiss);
-void    streamReadVarSlicePart(int streamID, int varID, int levelID, int varType, int start, size_t size, void *data, size_t *nmiss, int memtype);
+void    streamWriteVarSlice(int streamID, int varID, int levelID, const double data[], SizeType numMissVals);
+void    streamWriteVarSliceF(int streamID, int varID, int levelID, const float data[], SizeType numMissVals);
+void    streamReadVarSlicePart(int streamID, int varID, int levelID, int varType, int start, SizeType size, void *data, SizeType *numMissVals, int memtype);
 
 //      streamReadVarSlice: Read a horizontal slice of a variable
-void    streamReadVarSlice(int streamID, int varID, int levelID, double data[], size_t *nmiss);
-void    streamReadVarSliceF(int streamID, int varID, int levelID, float data[], size_t *nmiss);
+void    streamReadVarSlice(int streamID, int varID, int levelID, double data[], SizeType *numMissVals);
+void    streamReadVarSliceF(int streamID, int varID, int levelID, float data[], SizeType *numMissVals);
 
-void    streamWriteVarChunk(int streamID, int varID, const int rect[3][2], const double data[], size_t nmiss);
+void    streamWriteVarChunk(int streamID, int varID, const int rect[][2], const double data[], SizeType numMissVals);
+void    streamWriteVarChunkF(int streamID, int varID, const int rect[][2], const float data[], SizeType numMissVals);
 
 
-// STREAM record I/O routines (sequential access)
+// STREAM field I/O routines (sequential access)
 
-void    streamDefRecord(int streamID, int  varID, int  levelID);
-void    streamInqRecord(int streamID, int *varID, int *levelID);
-void    streamWriteRecord(int streamID, const double data[], size_t nmiss);
-void    streamWriteRecordF(int streamID, const float data[], size_t nmiss);
-void    streamReadRecord(int streamID, double data[], size_t *nmiss);
-void    streamReadRecordF(int streamID, float data[], size_t *nmiss);
-void    streamCopyRecord(int streamIDdest, int streamIDsrc);
+void    streamDefField(int streamID, int  varID, int  levelID);
+void    streamInqField(int streamID, int *varID, int *levelID);
+void    streamWriteField(int streamID, const double data[], SizeType numMissVals);
+void    streamWriteFieldF(int streamID, const float data[], SizeType numMissVals);
+void    streamReadField(int streamID, double data[], SizeType *numMissVals);
+void    streamReadFieldF(int streamID, float data[], SizeType *numMissVals);
+void    streamCopyField(int streamIDdest, int streamIDsrc);
 
 void    streamInqGRIBinfo(int streamID, int *intnum, float *fltnum, off_t *bignum);
 
@@ -441,8 +493,8 @@ char *cdiIterator_inqVariableName(CdiIterator *me);     // The caller is respons
 int cdiIterator_inqGridId(CdiIterator *me);             // The returned id is only valid until the next call to cdiIteratorNextField().
 
 // Reading data
-void cdiIterator_readField(CdiIterator *me, double data[], size_t *nmiss_optional);
-void cdiIterator_readFieldF(CdiIterator *me, float data[], size_t *nmiss_optional);
+void cdiIterator_readField(CdiIterator *me, double data[], SizeType *numMissVals_optional);
+void cdiIterator_readFieldF(CdiIterator *me, float data[], SizeType *numMissVals_optional);
 // TODO[NH]: Add functions to read partial fields.
 
 
@@ -478,9 +530,6 @@ void    vlistDestroy(int vlistID);
 //      vlistDuplicate: Duplicate a variable list
 int     vlistDuplicate(int vlistID);
 
-//      vlistCopy: Copy a variable list
-void    vlistCopy(int vlistID2, int vlistID1);
-
 //      vlistCopyFlag: Copy some entries of a variable list
 void    vlistCopyFlag(int vlistID2, int vlistID1);
 
@@ -500,18 +549,18 @@ int     vlistNumber(int vlistID);
 //      vlistNvars: Number of variables in a variable list
 int     vlistNvars(int vlistID);
 
-//      vlistNgrids: Number of grids in a variable list
-int     vlistNgrids(int vlistID);
+//      vlistNumGrids: Number of grids in a variable list
+int     vlistNumGrids(int vlistID);
 
-//      vlistNzaxis: Number of zaxis in a variable list
-int     vlistNzaxis(int vlistID);
+//      vlistNumZaxis: Number of zaxis in a variable list
+int     vlistNumZaxis(int vlistID);
 
 //      vlistNsubtypes: Number of subtypes in a variable list
 int     vlistNsubtypes(int vlistID);
 
 void    vlistDefNtsteps(int vlistID, int nts);
 int     vlistNtsteps(int vlistID);
-size_t  vlistGridsizeMax(int vlistID);
+SizeType vlistGridsizeMax(int vlistID);
 int     vlistGrid(int vlistID, int index);
 int     vlistGridIndex(int vlistID, int gridID);
 void    vlistChangeGridIndex(int vlistID, int index, int gridID);
@@ -520,7 +569,7 @@ int     vlistZaxis(int vlistID, int index);
 int     vlistZaxisIndex(int vlistID, int zaxisID);
 void    vlistChangeZaxisIndex(int vlistID, int index, int zaxisID);
 void    vlistChangeZaxis(int vlistID, int zaxisID1, int zaxisID2);
-int     vlistNrecs(int vlistID);
+int     vlistNumFields(int vlistID);
 int     vlistSubtype(int vlistID, int index);
 int     vlistSubtypeIndex(int vlistID, int subtypeID);
 
@@ -587,12 +636,12 @@ void    vlistDefVarDatatype(int vlistID, int varID, int datatype);
 //      vlistInqVarDatatype: Get the data type of a Variable
 int     vlistInqVarDatatype(int vlistID, int varID);
 
-void    vlistDefVarChunkType(int vlistID, int varID, int chunktype);
-int     vlistInqVarChunkType(int vlistID, int varID);
-
 void    vlistDefVarXYZ(int vlistID, int varID, int xyz);
 int     vlistInqVarXYZ(int vlistID, int varID);
 
+void    vlistDefVarNSB(int vlistID, int varID, int nsb);
+int     vlistInqVarNSB(int vlistID, int varID);
+
 int     vlistInqVarNumber(int vlistID, int varID);
 
 void    vlistDefVarInstitut(int vlistID, int varID, int instID);
@@ -635,18 +684,7 @@ void    vlistDefVarMissval(int vlistID, int varID, double missval);
 //      vlistInqVarMissval: Get the missing value of a Variable
 double  vlistInqVarMissval(int vlistID, int varID);
 
-//      vlistDefVarExtra: Define extra information of a Variable
-void    vlistDefVarExtra(int vlistID, int varID, const char *extra);
-
-//      vlistInqVarExtra: Get extra information of a Variable
-void    vlistInqVarExtra(int vlistID, int varID, char *extra);
-
-void    vlistDefVarScalefactor(int vlistID, int varID, double scalefactor);
-double  vlistInqVarScalefactor(int vlistID, int varID);
-void    vlistDefVarAddoffset(int vlistID, int varID, double addoffset);
-double  vlistInqVarAddoffset(int vlistID, int varID);
-
-size_t  vlistInqVarSize(int vlistID, int varID);
+SizeType vlistInqVarSize(int vlistID, int varID);
 
 void    vlistDefIndex(int vlistID, int varID, int levID, int index);
 int     vlistInqIndex(int vlistID, int varID, int levID);
@@ -715,7 +753,7 @@ void    gridDefMask(int gridID, const int mask[]);
 int     gridInqMask(int gridID, int mask[]);
 
 //      gridCreate: Create a horizontal Grid
-int     gridCreate(int gridtype, size_t size);
+int     gridCreate(int gridtype, SizeType size);
 
 //      gridDestroy: Destroy a horizontal Grid
 void    gridDestroy(int gridID);
@@ -736,19 +774,19 @@ int     gridInqProjType(int gridID);
 int     gridInqType(int gridID);
 
 //      gridInqSize: Get the size of a Grid
-size_t  gridInqSize(int gridID);
+SizeType gridInqSize(int gridID);
 
 //      gridDefXsize: Define the size of a X-axis
-void    gridDefXsize(int gridID, size_t xsize);
+void    gridDefXsize(int gridID, SizeType xsize);
 
 //      gridInqXsize: Get the size of a X-axis
-size_t  gridInqXsize(int gridID);
+SizeType gridInqXsize(int gridID);
 
 //      gridDefYsize: Define the size of a Y-axis
-void    gridDefYsize(int gridID, size_t ysize);
+void    gridDefYsize(int gridID, SizeType ysize);
 
 //      gridInqYsize: Get the size of a Y-axis
-size_t  gridInqYsize(int gridID);
+SizeType gridInqYsize(int gridID);
 
 //      gridDefNP: Define the number of parallels between a pole and the equator
 void    gridDefNP(int gridID, int np);
@@ -760,27 +798,29 @@ int     gridInqNP(int gridID);
 void    gridDefXvals(int gridID, const double xvals[]);
 
 //      gridInqXvals: Get all values of a X-axis
-size_t  gridInqXvals(int gridID, double xvals[]);
-size_t  gridInqXvalsPart(int gridID, int start, size_t size, double xvals[]);
+SizeType gridInqXvals(int gridID, double xvals[]);
+SizeType gridInqXvalsPart(int gridID, int start, SizeType size, double xvals[]);
+const double *gridInqXvalsPtr(int gridID);
 
 //      gridInqXIsc: Find out whether X-coordinate is of type CHAR
 int     gridInqXIsc(int gridID);
 
 //      gridInqXCvals: Get strings from X-axis in case grid is of type GRID_CHARXY
-size_t  gridInqXCvals(int gridID, char *xcvals[]);
+SizeType gridInqXCvals(int gridID, char *xcvals[]);
 
 //      gridDefYvals: Define the values of a Y-axis
 void    gridDefYvals(int gridID, const double yvals[]);
 
 //      gridInqYvals: Get all values of a Y-axis
-size_t  gridInqYvals(int gridID, double yvals[]);
-size_t  gridInqYvalsPart(int gridID, int start, size_t size, double yvals[]);
+SizeType gridInqYvals(int gridID, double yvals[]);
+SizeType gridInqYvalsPart(int gridID, int start, SizeType size, double yvals[]);
+const double *gridInqYvalsPtr(int gridID);
 
 //      gridInqYIsc: Find out whether Y-coordinate is of type CHAR
 int     gridInqYIsc(int gridID);
 
 //      gridInqYCvals: Get strings from Y-axis in case grid is of type GRID_CHARXY
-size_t  gridInqYCvals(int gridID, char *ycvals[]);
+SizeType gridInqYCvals(int gridID, char *ycvals[]);
 
 // CDI var keys
 
@@ -790,16 +830,23 @@ size_t  gridInqYCvals(int gridID, char *ycvals[]);
 #define  CDI_KEY_STDNAME                       944  // CF Standard name of the variable
 #define  CDI_KEY_UNITS                         945  // Units of the variable
 #define  CDI_KEY_DATATYPE                      946  // Data type
-#define  CDI_KEY_REFERENCEURI                  965  // Reference URI to grid file
+#define  CDI_KEY_REFERENCEURI                  947  // Reference URI to grid file
+#define  CDI_KEY_CHUNKS                        948  // Chunks
 
 // Integer keys
-#define  CDI_KEY_NUMBEROFGRIDUSED              963  // GRIB2 numberOfGridUsed
-#define  CDI_KEY_NUMBEROFGRIDINREFERENCE       964  // GRIB2 numberOfGridInReference
-#define  CDI_KEY_NUMBEROFVGRIDUSED             961  // GRIB2 numberOfVGridUsed
-#define  CDI_KEY_NLEV                          962  // GRIB2 nlev
+#define  CDI_KEY_NUMBEROFGRIDUSED              961  // GRIB2 numberOfGridUsed
+#define  CDI_KEY_NUMBEROFGRIDINREFERENCE       962  // GRIB2 numberOfGridInReference
+#define  CDI_KEY_NUMBEROFVGRIDUSED             963  // GRIB2 numberOfVGridUsed
+#define  CDI_KEY_NLEV                          964  // GRIB2 nlev
+#define  CDI_KEY_CHUNKTYPE                     965  // ChunkType: CDI_CHUNK_AUTO/CDI_CHUNK_GRID/CDI_CHUNK_LINES
+#define  CDI_KEY_CHUNKSIZE                     966  // ChunkSize
 
 // Floating point keys
 #define  CDI_KEY_MISSVAL                       701  // Missing value
+#define  CDI_KEY_ADDOFFSET                     702  // Add offset
+#define  CDI_KEY_SCALEFACTOR                   703  // Scale factor
+#define  CDI_KEY_FILTERSPEC_IN                 720  // NetCDF4 filter specification read in
+#define  CDI_KEY_FILTERSPEC                    721  // NetCDF4 filter specification
 
 // Byte array keys
 #define  CDI_KEY_UUID                          960  // UUID for grid/Z-axis reference [size: CDI_UUID_SIZE]
@@ -918,10 +965,10 @@ void    gridDefDatatype(int gridID, int datatype);
 int     gridInqDatatype(int gridID);
 
 //      gridInqXval: Get one value of a X-axis
-double  gridInqXval(int gridID, size_t index);
+double  gridInqXval(int gridID, SizeType index);
 
 //      gridInqYval: Get one value of a Y-axis
-double  gridInqYval(int gridID, size_t index);
+double  gridInqYval(int gridID, SizeType index);
 
 double  gridInqXinc(int gridID);
 double  gridInqYinc(int gridID);
@@ -979,15 +1026,17 @@ int     gridInqNvertex(int gridID);
 void    gridDefXbounds(int gridID, const double xbounds[]);
 
 //      gridInqXbounds: Get the bounds of a X-axis
-size_t  gridInqXbounds(int gridID, double xbounds[]);
-size_t  gridInqXboundsPart(int gridID, int start, size_t size, double xbounds[]);
+SizeType gridInqXbounds(int gridID, double xbounds[]);
+SizeType gridInqXboundsPart(int gridID, int start, SizeType size, double xbounds[]);
+const double *gridInqXboundsPtr(int gridID);
 
 //      gridDefYbounds: Define the bounds of a Y-axis
 void    gridDefYbounds(int gridID, const double ybounds[]);
 
 //      gridInqYbounds: Get the bounds of a Y-axis
-size_t  gridInqYbounds(int gridID, double ybounds[]);
-size_t  gridInqYboundsPart(int gridID, int start, size_t size, double ybounds[]);
+SizeType gridInqYbounds(int gridID, double ybounds[]);
+SizeType gridInqYboundsPart(int gridID, int start, SizeType size, double ybounds[]);
+const double *gridInqYboundsPtr(int gridID);
 
 void    gridDefReducedPoints(int gridID, int reducedPointsSize, const int reducedPoints[]);
 void    gridInqReducedPoints(int gridID, int reducedPoints[]);
@@ -1115,51 +1164,39 @@ void    taxisDefType(int taxisID, int taxistype);
 int     taxisInqType(int taxisID);
 
 //      taxisDefVdate: Define the verification date
-void    taxisDefVdate(int taxisID, int64_t date);
+void    taxisDefVdate(int taxisID, int date);
 
 //      taxisDefVtime: Define the verification time
 void    taxisDefVtime(int taxisID, int time);
 
 //      taxisInqVdate: Get the verification date
-int64_t taxisInqVdate(int taxisID);
+int     taxisInqVdate(int taxisID);
 
 //      taxisInqVtime: Get the verification time
 int     taxisInqVtime(int taxisID);
 
 //      taxisDefRdate: Define the reference date
-void    taxisDefRdate(int taxisID, int64_t date);
+void    taxisDefRdate(int taxisID, int date);
 
 //      taxisDefRtime: Define the reference time
 void    taxisDefRtime(int taxisID, int time);
 
 //      taxisInqRdate: Get the reference date
-int64_t taxisInqRdate(int taxisID);
+int     taxisInqRdate(int taxisID);
 
 //      taxisInqRtime: Get the reference time
 int     taxisInqRtime(int taxisID);
 
-//      taxisDefFdate: Define the forecast reference date
-void    taxisDefFdate(int taxisID, int64_t date);
-
-//      taxisDefFtime: Define the forecast reference time
-void    taxisDefFtime(int taxisID, int time);
-
-//      taxisInqFdate: Get the forecast reference date
-int64_t taxisInqFdate(int taxisID);
-
-//      taxisInqFtime: Get the forecast reference time
-int     taxisInqFtime(int taxisID);
-
 int     taxisHasBounds(int taxisID);
 void    taxisWithBounds(int taxisID);
 
 void    taxisDeleteBounds(int taxisID);
 
-void    taxisDefVdateBounds(int taxisID, int64_t vdate_lb, int64_t vdate_ub);
+void    taxisDefVdateBounds(int taxisID, int vdate_lb, int vdate_ub);
 
 void    taxisDefVtimeBounds(int taxisID, int vtime_lb, int vtime_ub);
 
-void    taxisInqVdateBounds(int taxisID, int64_t *vdate_lb, int64_t *vdate_ub);
+void    taxisInqVdateBounds(int taxisID, int *vdate_lb, int *vdate_ub);
 
 void    taxisInqVtimeBounds(int taxisID, int *vtime_lb, int *vtime_ub);
 
@@ -1263,16 +1300,99 @@ int     vlistInqVarSubtype(int vlistID, int varID);
 
 void gribapiLibraryVersion(int *major_version, int *minor_version, int *revision_version);
 
+// Compatibility functions for release 1.8.3 (obsolete functions)
+void zaxisDefLtype(int zaxisID, int ltype);
+int  vlistInqVarTypeOfGeneratingProcess(int vlistID, int varID);
+void vlistDefVarTypeOfGeneratingProcess(int vlistID, int varID, int typeOfGeneratingProcess);
+void vlistDefVarProductDefinitionTemplate(int vlistID, int varID, int productDefinitionTemplate);
+
 #ifdef __cplusplus
 }
 #endif
 
+// End of fortran interface
 //FINT_OFF  <--- don't change or remove this line!!!
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+// CDI query interface
+
+typedef struct
+{
+  int numEntries;
+  // Names
+  int numNames;
+  bool *namesFound;
+  char **names;
+  // Grid cell indices
+  int numCellidx;
+  bool *cellidxFound;
+  size_t *cellidx;
+  // Level indices
+  int numLevidx;
+  bool *levidxFound;
+  int *levidx;
+  // Time step indices
+  int numStepidx;
+  bool *stepidxFound;
+  int *stepidx;
+} CdiQuery;
+
+CdiQuery *cdiQueryCreate(void);
+CdiQuery *cdiQueryClone(const CdiQuery *query);
+void cdiQueryDelete(CdiQuery *query);
+void cdiQuerySetNames(CdiQuery *query, int numNames, char **names);
+void cdiQuerySetCellidx(CdiQuery *query, int numCellidx, size_t *cellidx);
+void cdiQuerySetLevidx(CdiQuery *query, int numLevidx, int *levidx);
+void cdiQuerySetStepidx(CdiQuery *query, int numStepidx, int *stepidx);
+size_t cdiQueryGetCellidx(const CdiQuery *query, int index);
+int cdiQueryName(CdiQuery *query, const char *name);
+int cdiQueryCellidx(CdiQuery *query, size_t cellidx);
+int cdiQueryLevidx(CdiQuery *query, int levidx);
+int cdiQueryStepidx(CdiQuery *query, int stepidx);
+int cdiQueryNumNames(const CdiQuery *query);
+int cdiQueryNumCellidx(const CdiQuery *query);
+int cdiQueryNumStepidx(const CdiQuery *query);
+int cdiQueryNumEntries(const CdiQuery *query);
+int cdiQueryNumEntriesFound(const CdiQuery *query);
+void cdiQueryPrint(const CdiQuery *query);
+void cdiQueryPrintEntriesNotFound(const CdiQuery *query);
+
+// streamOpenReadQuery: Open a dataset for reading and apply query
+int streamOpenReadQuery(const char *path, CdiQuery *query);
+
+// CDI interface for paraview vtkCDIReader.cxx
+
+#include "cdi_datetime.h"
+
+// taxisDefRdatetime: Define the reference date/time
+void taxisDefRdatetime(int taxisID, CdiDateTime rDateTime);
+// taxisInqRdatetime: Get the reference date/time
+CdiDateTime taxisInqRdatetime(int taxisID);
+// taxisDefFdatetime: Define the forecast reference date/time
+void taxisDefFdatetime(int taxisID, CdiDateTime fDateTime);
+// taxisInqFdatetime: Get the forecast reference date/time
+CdiDateTime taxisInqFdatetime(int taxisID);
+// taxisDefVdatetime: Define the verification date/time
+void taxisDefVdatetime(int taxisID, CdiDateTime vDateTime);
+// taxisInqVdatetime: Get the verification date/time
+CdiDateTime taxisInqVdatetime(int taxisID);
+void taxisDefVdatetimeBounds(int taxisID, CdiDateTime vDateTime_lb, CdiDateTime vDateTime_ub);
+void taxisInqVdatetimeBounds(int taxisID, CdiDateTime *vDateTime_lb, CdiDateTime *vDateTime_ub);
+
+// date format:  YYYYMMDD
+// time format:    hhmmss
+
+int64_t date_to_julday(int calendar, int64_t date);  // Used in paraview vtkCDIReader.cxx
+int64_t julday_to_date(int calendar, int64_t julday);
+
+int time_to_sec(int time);                           // Used in paraview vtkCDIReader.cxx
+int sec_to_time(int secofday);
+
+// CDI projection parameter interface
+
 struct CDI_GridProjParams
 {
   double mv;     // Missing value
@@ -1288,6 +1408,10 @@ struct CDI_GridProjParams
   double yval_0; // Latitude of the first grid point in degree (optional)
   double x_0;    // False easting (optional)
   double y_0;    // False northing (optional)
+  double x_SP;   // Longitude of southern pole
+  double y_SP;   // Latitude of southern pole
+  int nside;     // HEALPix number of points along a side (number of data points should be = 12 * nside * nside)
+  int order;     // HEALPix ordering convention (0:ring; 1:nested)
 };
 
 void gridProjParamsInit(struct CDI_GridProjParams *gridProjParams);
@@ -1300,17 +1424,26 @@ int  gridInqParamsLCC(int gridID, struct CDI_GridProjParams *gridProjParams);
 void gridDefParamsSTERE(int gridID, struct CDI_GridProjParams gridProjParams);
 int  gridInqParamsSTERE(int gridID, struct CDI_GridProjParams *gridProjParams);
 
+// HEALPix grid
+void gridDefParamsHEALPIX(int gridID, struct CDI_GridProjParams gridProjParams);
+int  gridInqParamsHEALPIX(int gridID, struct CDI_GridProjParams *gridProjParams);
+
 #define HAVE_CDI_PROJ_FUNCS 1
 extern int (*proj_lonlat_to_lcc_func)(struct CDI_GridProjParams gpp, size_t, double*, double*);
-extern int (*proj_lcc_to_lonlat_func)(struct CDI_GridProjParams *gpp, double, double, size_t, double*, double*);
+extern int (*proj_lcc_to_lonlat_func)(struct CDI_GridProjParams gpp, double, double, size_t, double*, double*);
 extern int (*proj_lonlat_to_stere_func)(struct CDI_GridProjParams gpp, size_t, double*, double*);
-extern int (*proj_stere_to_lonlat_func)(struct CDI_GridProjParams *gpp, double, double, size_t, double*, double*);
+extern int (*proj_stere_to_lonlat_func)(struct CDI_GridProjParams gpp, double, double, size_t, double*, double*);
+
+// Used on CDO remap_scrip_io.cc
+void cdf_def_var_filter(int ncid, int ncvarID, const char *filterSpec);
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif  /* CDI_H_ */
+// clang-format on
+
+#endif /* CDI_H_ */
 /*
  * Local Variables:
  * c-file-style: "Java"
