@@ -1,48 +1,6 @@
-/**
- * @file sphere_part.c
- *
- * @copyright Copyright  (C)  2014 Moritz Hanke <hanke@dkrz.de>
- *                                 Thomas Jahns <jahns@dkrz.de>
- *
- * @version 1.0
- * @author Moritz Hanke <hanke@dkrz.de>
- *         Thomas Jahns <jahns@dkrz.de>
- */
-/*
- * Keywords:
- * Maintainer: Moritz Hanke <hanke@dkrz.de>
- *             Rene Redler <rene.redler@mpimet.mpg.de>
- * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
- *
- * This file is part of YAC.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are  permitted provided that the following conditions are
- * met:
- *
- * Redistributions of source code must retain the above copyright notice,
- * this list of conditions and the following disclaimer.
- *
- * Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in the
- * documentation and/or other materials provided with the distribution.
- *
- * Neither the name of the DKRZ GmbH nor the names of its contributors
- * may be used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
- * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
- * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
- * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// Copyright (c) 2024 The YAC Authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifdef HAVE_CONFIG_H
 // Get the definition of the 'restrict' keyword.
@@ -58,9 +16,9 @@
 
 #include "sphere_part.h"
 #include "geometry.h"
-#include "grid.h"
+#include "basic_grid.h"
 #include "interval_tree.h"
-#include "utils.h"
+#include "utils_core.h"
 #include "ensure_array_size.h"
 
 union I_list {
@@ -202,7 +160,7 @@ static inline void compute_gc_norm_vector(
      balance_point[2] = prev_gc_norm_vector[1];
    }
 
-   crossproduct_ld(
+   crossproduct_kahan(
       balance_point, prev_gc_norm_vector, gc_norm_vector);
 
   if ((fabs(gc_norm_vector[0]) > 1e-9) ||
@@ -401,9 +359,9 @@ static void partition_data (
 
             // project the base vector of the current bounding circle onto the
             // current partitioning great circle
-            crossproduct_ld(parent_node->gc_norm_vector,
+            crossproduct_kahan(parent_node->gc_norm_vector,
                             curr_bnd_circle.base_vector, GCp);
-            crossproduct_ld(GCp, parent_node->gc_norm_vector, bVp);
+            crossproduct_kahan(GCp, parent_node->gc_norm_vector, bVp);
             YAC_ASSERT(
               (fabs(bVp[0]) > 1e-9) || (fabs(bVp[1]) > 1e-9) ||
               (fabs(bVp[2]) > 1e-9),
@@ -626,7 +584,7 @@ static inline int compare_points_int32_coord(
 
 static struct point_id_xyz *
   get_unique_points(
-    size_t * num_points, coordinate_pointer coordinates_xyz,
+    size_t * num_points, yac_const_coordinate_pointer coordinates_xyz,
     yac_int const * ids, int const * mask) {
 
   struct point_id_xyz_int32 * points_int32 =
@@ -699,7 +657,8 @@ static struct point_id_xyz *
 }
 
 struct point_sphere_part_search * yac_point_sphere_part_search_new (
-  size_t num_points, coordinate_pointer coordinates_xyz, yac_int const * ids) {
+  size_t num_points, yac_const_coordinate_pointer coordinates_xyz,
+  yac_int const * ids) {
 
   if (num_points == 0) return NULL;
 
@@ -724,7 +683,7 @@ struct point_sphere_part_search * yac_point_sphere_part_search_new (
 }
 
 struct point_sphere_part_search * yac_point_sphere_part_search_mask_new (
-  size_t num_points, coordinate_pointer coordinates_xyz,
+  size_t num_points, yac_const_coordinate_pointer coordinates_xyz,
   yac_int const * ids, int const * mask) {
 
   if (num_points == 0) return NULL;
@@ -778,8 +737,8 @@ static void search_bnd_circle_I_node(
 
     // project the base vector of the current bounding circle onto the
     // current great circle
-    crossproduct_ld(node->gc_norm_vector, bnd_circle.base_vector, GCp);
-    crossproduct_ld(GCp, node->gc_norm_vector, bVp);
+    crossproduct_kahan(node->gc_norm_vector, bnd_circle.base_vector, GCp);
+    crossproduct_kahan(GCp, node->gc_norm_vector, bVp);
     YAC_ASSERT(
       (fabs(bVp[0]) > 1e-9) || (fabs(bVp[1]) > 1e-9) || (fabs(bVp[2]) > 1e-9),
       "ERROR(search_bnd_circle_I_node): "
@@ -1815,8 +1774,8 @@ static void search_point(struct sphere_part_node * node,
 
          // project the point onto the current great circle
          double GCp[3], bVp[3];
-         crossproduct_ld(node->gc_norm_vector, point, GCp);
-         crossproduct_ld(GCp, node->gc_norm_vector, bVp);
+         crossproduct_kahan(node->gc_norm_vector, point, GCp);
+         crossproduct_kahan(GCp, node->gc_norm_vector, bVp);
 
          struct interval search_interval;
          // if the projected point does not coincide with the norm vector of
@@ -1862,7 +1821,7 @@ static void search_point(struct sphere_part_node * node,
 }
 
 void yac_bnd_sphere_part_search_do_point_search(
-  struct bnd_sphere_part_search * search, coordinate_pointer coordinates_xyz,
+  struct bnd_sphere_part_search * search, yac_coordinate_pointer coordinates_xyz,
   size_t count, size_t ** cells, size_t * num_cells_per_coordinate) {
 
   struct sphere_part_node * base_node = &(search->base_node);
